import React from 'react';

function isEqual(a, b) {
    if (a === b)
        return true;
    if (a && b && typeof a == 'object' && typeof b == 'object') {
        if (a.constructor !== b.constructor)
            return false;
        if (Array.isArray(a)) {
            if (a.length != b.length)
                return false;
            for (let i = 0; i < a.length; ++i) {
                if (!isEqual(a[i], b[i]))
                    return false;
            }
            return true;
        }
        if (a.constructor === RegExp)
            return a.source === b.source && a.flags === b.flags;
        if (a.valueOf !== Object.prototype.valueOf)
            return a.valueOf() === b.valueOf();
        if (a.toString !== Object.prototype.toString)
            return a.toString() === b.toString();
        const keys = Object.keys(a);
        if (keys.length !== Object.keys(b).length)
            return false;
        for (let i = 0; i < keys.length; ++i) {
            if (!Object.prototype.hasOwnProperty.call(b, keys[i]))
                return false;
        }
        for (let i = 0; i < keys.length; ++i) {
            const key = keys[i];
            if (key === '_owner' && a.$$typeof)
                continue;
            if (!isEqual(a[key], b[key]))
                return false;
        }
        return true;
    }
    return a !== a && b !== b;
}

function useDeepCompareCallback(fn, deps) {
    const signal = React.useRef(0);
    const prevDeps = React.useRef(deps);
    if (!isEqual(prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    return React.useCallback(fn, [signal.current]);
}

function useDeepCompareEffect(fn, deps) {
    const signal = React.useRef(0);
    const prevDeps = React.useRef(deps);
    if (!isEqual(prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    React.useEffect(fn, [signal.current]);
}

function useDeepCompareMemo(fn, deps) {
    const signal = React.useRef(0);
    const prevDeps = React.useRef(deps);
    if (!isEqual(prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    return React.useMemo(fn, [signal.current]);
}

function useEventCallback(handler) {
    const handlerRef = React.useRef(handler);
    React.useLayoutEffect(() => {
        handlerRef.current = handler;
    });
    return React.useCallback((...args) => {
        const handle = handlerRef.current;
        return handle(...args);
    }, []);
}

function useInterval(callback, duration) {
    const callbackRef = React.useRef(callback);
    React.useEffect(() => {
        callbackRef.current = callback;
    }, [callback]);
    React.useEffect(() => {
        const tick = () => {
            if (callbackRef.current === undefined)
                return;
            callbackRef.current();
        };
        const intervalId = setInterval(tick, duration);
        return () => clearInterval(intervalId);
    }, [duration]);
}

function usePreviousState(value) {
    const ref = React.useRef(value);
    React.useEffect(() => {
        ref.current = value;
    }, [value]);
    return ref.current;
}

function useRefreshRef(value) {
    const ref = React.useRef(value);
    ref.current = value;
    return ref;
}

function useSyncState(initialState) {
    const [state, setState] = React.useState(initialState);
    React.useEffect(() => setState(initialState), [initialState]);
    return [state, setState];
}

export { isEqual, useDeepCompareCallback, useDeepCompareEffect, useDeepCompareMemo, useEventCallback, useInterval, usePreviousState, useRefreshRef, useSyncState };
//# sourceMappingURL=index.mjs.map
